from __future__ import absolute_import

import urllib3
import unittest
import requests

import tests.apps.flask_app
from ..helpers import testenv
from instana.singletons import agent, tracer


class TestUrllib3(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.http = urllib3.PoolManager()
        self.recorder = tracer.recorder
        self.recorder.clear_spans()

    def tearDown(self):
        """ Do nothing for now """
        return None

    def test_vanilla_requests(self):
        r = self.http.request('GET', testenv["wsgi_server"] + '/')
        self.assertEqual(r.status, 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))

    def test_get_request(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_get_request_with_query(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/?one=1&two=2')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span.data["http"]["url"])
        self.assertTrue(urllib3_span.data["http"]["params"] in ["one=1&two=2", "two=2&one=1"] )
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_get_request_with_alt_query(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/', fields={'one': '1', 'two': 2})

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span.data["http"]["url"])
        self.assertTrue(urllib3_span.data["http"]["params"] in ["one=1&two=2", "two=2&one=1"] )
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_put_request(self):
        with tracer.start_active_span('test'):
            r = self.http.request('PUT', testenv["wsgi_server"] + '/notfound')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(404, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/notfound', wsgi_span.data["http"]["url"])
        self.assertEqual('PUT', wsgi_span.data["http"]["method"])
        self.assertEqual(404, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(404, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/notfound", urllib3_span.data["http"]["url"])
        self.assertEqual("PUT", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_301_redirect(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/301')

        spans = self.recorder.queued_spans()
        self.assertEqual(5, len(spans))

        wsgi_span2 = spans[0]
        urllib3_span2 = spans[1]
        wsgi_span1 = spans[2]
        urllib3_span1 = spans[3]
        test_span = spans[4]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span1.t)
        self.assertEqual(traceId, wsgi_span1.t)
        self.assertEqual(traceId, urllib3_span2.t)
        self.assertEqual(traceId, wsgi_span2.t)

        # Parent relationships
        self.assertEqual(urllib3_span1.p, test_span.s)
        self.assertEqual(wsgi_span1.p, urllib3_span1.s)
        self.assertEqual(urllib3_span2.p, test_span.s)
        self.assertEqual(wsgi_span2.p, urllib3_span2.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span1.ec)
        self.assertIsNone(wsgi_span1.ec)
        self.assertIsNone(urllib3_span2.ec)
        self.assertIsNone(wsgi_span2.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span1.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span1.data["http"]["host"])
        self.assertEqual('/', wsgi_span1.data["http"]["url"])
        self.assertEqual('GET', wsgi_span1.data["http"]["method"])
        self.assertEqual(200, wsgi_span1.data["http"]["status"])
        self.assertIsNone(wsgi_span1.data["http"]["error"])
        self.assertIsNotNone(wsgi_span1.stack)
        self.assertEqual(2, len(wsgi_span1.stack))

        self.assertEqual("wsgi", wsgi_span2.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span2.data["http"]["host"])
        self.assertEqual('/301', wsgi_span2.data["http"]["url"])
        self.assertEqual('GET', wsgi_span2.data["http"]["method"])
        self.assertEqual(301, wsgi_span2.data["http"]["status"])
        self.assertIsNone(wsgi_span2.data["http"]["error"])
        self.assertIsNotNone(wsgi_span2.stack)
        self.assertEqual(2, len(wsgi_span2.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span1.n)
        self.assertEqual(200, urllib3_span1.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span1.data["http"]["url"])
        self.assertEqual("GET", urllib3_span1.data["http"]["method"])
        self.assertIsNotNone(urllib3_span1.stack)
        self.assertTrue(type(urllib3_span1.stack) is list)
        self.assertTrue(len(urllib3_span1.stack) > 1)

        self.assertEqual("urllib3", urllib3_span2.n)
        self.assertEqual(301, urllib3_span2.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/301", urllib3_span2.data["http"]["url"])
        self.assertEqual("GET", urllib3_span2.data["http"]["method"])
        self.assertIsNotNone(urllib3_span2.stack)
        self.assertTrue(type(urllib3_span2.stack) is list)
        self.assertTrue(len(urllib3_span2.stack) > 1)

    def test_302_redirect(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/302')

        spans = self.recorder.queued_spans()
        self.assertEqual(5, len(spans))

        wsgi_span2 = spans[0]
        urllib3_span2 = spans[1]
        wsgi_span1 = spans[2]
        urllib3_span1 = spans[3]
        test_span = spans[4]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span1.t)
        self.assertEqual(traceId, wsgi_span1.t)
        self.assertEqual(traceId, urllib3_span2.t)
        self.assertEqual(traceId, wsgi_span2.t)

        # Parent relationships
        self.assertEqual(urllib3_span1.p, test_span.s)
        self.assertEqual(wsgi_span1.p, urllib3_span1.s)
        self.assertEqual(urllib3_span2.p, test_span.s)
        self.assertEqual(wsgi_span2.p, urllib3_span2.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span1.ec)
        self.assertIsNone(wsgi_span1.ec)
        self.assertIsNone(urllib3_span2.ec)
        self.assertIsNone(wsgi_span2.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span1.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span1.data["http"]["host"])
        self.assertEqual('/', wsgi_span1.data["http"]["url"])
        self.assertEqual('GET', wsgi_span1.data["http"]["method"])
        self.assertEqual(200, wsgi_span1.data["http"]["status"])
        self.assertIsNone(wsgi_span1.data["http"]["error"])
        self.assertIsNotNone(wsgi_span1.stack)
        self.assertEqual(2, len(wsgi_span1.stack))

        self.assertEqual("wsgi", wsgi_span2.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span2.data["http"]["host"])
        self.assertEqual('/302', wsgi_span2.data["http"]["url"])
        self.assertEqual('GET', wsgi_span2.data["http"]["method"])
        self.assertEqual(302, wsgi_span2.data["http"]["status"])
        self.assertIsNone(wsgi_span2.data["http"]["error"])
        self.assertIsNotNone(wsgi_span2.stack)
        self.assertEqual(2, len(wsgi_span2.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span1.n)
        self.assertEqual(200, urllib3_span1.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span1.data["http"]["url"])
        self.assertEqual("GET", urllib3_span1.data["http"]["method"])
        self.assertIsNotNone(urllib3_span1.stack)
        self.assertTrue(type(urllib3_span1.stack) is list)
        self.assertTrue(len(urllib3_span1.stack) > 1)

        self.assertEqual("urllib3", urllib3_span2.n)
        self.assertEqual(302, urllib3_span2.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/302", urllib3_span2.data["http"]["url"])
        self.assertEqual("GET", urllib3_span2.data["http"]["method"])
        self.assertIsNotNone(urllib3_span2.stack)
        self.assertTrue(type(urllib3_span2.stack) is list)
        self.assertTrue(len(urllib3_span2.stack) > 1)

    def test_5xx_request(self):
        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/504')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(504, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span.t)
        self.assertEqual(traceId, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertEqual(1, urllib3_span.ec)
        self.assertEqual(1, wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/504', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(504, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(504, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/504", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_exception_logging(self):
        with tracer.start_active_span('test'):
            try:
                r = self.http.request('GET', testenv["wsgi_server"] + '/exception')
            except Exception:
                pass

        spans = self.recorder.queued_spans()
        self.assertEqual(4, len(spans))

        wsgi_span = spans[1]
        urllib3_span = spans[2]
        test_span = spans[3]

        assert(r)
        self.assertEqual(500, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span.t)
        self.assertEqual(traceId, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertEqual(1, urllib3_span.ec)
        self.assertEqual(1, wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/exception', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(500, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(500, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/exception", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_client_error(self):
        r = None
        with tracer.start_active_span('test'):
            try:
                r = self.http.request('GET', 'http://doesnotexist.asdf:5000/504',
                                      retries=False,
                                      timeout=urllib3.Timeout(connect=0.5, read=0.5))
            except Exception:
                pass

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        urllib3_span = spans[0]
        test_span = spans[1]

        self.assertIsNone(r)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span.t)

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertIsNone(urllib3_span.data["http"]["status"])
        self.assertEqual("http://doesnotexist.asdf:5000/504", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertEqual(1, urllib3_span.ec)

    def test_requestspkg_get(self):
        self.recorder.clear_spans()
        
        with tracer.start_active_span('test'):
            r = requests.get(testenv["wsgi_server"] + '/', timeout=2)

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status_code)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_requestspkg_get_with_custom_headers(self):
        my_custom_headers = dict()
        my_custom_headers['X-PGL-1'] = '1'

        with tracer.start_active_span('test'):
            r = requests.get(testenv["wsgi_server"] + '/', timeout=2, headers=my_custom_headers)

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status_code)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_requestspkg_put(self):
        with tracer.start_active_span('test'):
            r = requests.put(testenv["wsgi_server"] + '/notfound')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        self.assertEqual(404, r.status_code)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/notfound', wsgi_span.data["http"]["url"])
        self.assertEqual('PUT', wsgi_span.data["http"]["method"])
        self.assertEqual(404, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(404, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/notfound", urllib3_span.data["http"]["url"])
        self.assertEqual("PUT", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)

    def test_response_header_capture(self):
        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This']

        with tracer.start_active_span('test'):
            r = self.http.request('GET', testenv["wsgi_server"] + '/response_headers')

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        wsgi_span = spans[0]
        urllib3_span = spans[1]
        test_span = spans[2]

        assert(r)
        self.assertEqual(200, r.status)
        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, wsgi_span.t)

        # Parent relationships
        self.assertEqual(urllib3_span.p, test_span.s)
        self.assertEqual(wsgi_span.p, urllib3_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(urllib3_span.ec)
        self.assertIsNone(wsgi_span.ec)

        # wsgi
        self.assertEqual("wsgi", wsgi_span.n)
        self.assertEqual('127.0.0.1:' + str(testenv["wsgi_port"]), wsgi_span.data["http"]["host"])
        self.assertEqual('/response_headers', wsgi_span.data["http"]["url"])
        self.assertEqual('GET', wsgi_span.data["http"]["method"])
        self.assertEqual(200, wsgi_span.data["http"]["status"])
        self.assertIsNone(wsgi_span.data["http"]["error"])
        self.assertIsNotNone(wsgi_span.stack)
        self.assertEqual(2, len(wsgi_span.stack))

        # urllib3
        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual("urllib3", urllib3_span.n)
        self.assertEqual(200, urllib3_span.data["http"]["status"])
        self.assertEqual(testenv["wsgi_server"] + "/response_headers", urllib3_span.data["http"]["url"])
        self.assertEqual("GET", urllib3_span.data["http"]["method"])
        self.assertIsNotNone(urllib3_span.stack)
        self.assertTrue(type(urllib3_span.stack) is list)
        self.assertTrue(len(urllib3_span.stack) > 1)
        self.assertTrue('http.X-Capture-This' in urllib3_span.data["custom"]["tags"])

        agent.options.extra_http_headers = original_extra_http_headers




import re
import sys
import json
import time
import unittest
import opentracing
from uuid import UUID
from instana.util import to_json
from instana.singletons import agent, tracer
from ..helpers import get_first_span_by_filter

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


class TestOTSpan(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        agent.options.service_name = None
        opentracing.tracer = tracer
        recorder = opentracing.tracer.recorder
        recorder.clear_spans()

    def tearDown(self):
        """ Do nothing for now """
        return None

    def test_span_interface(self):
        span = opentracing.tracer.start_span("blah")
        assert hasattr(span, "finish")
        assert hasattr(span, "set_tag")
        assert hasattr(span, "tags")
        assert hasattr(span, "operation_name")
        assert hasattr(span, "set_baggage_item")
        assert hasattr(span, "get_baggage_item")
        assert hasattr(span, "context")
        assert hasattr(span, "log")

    def test_span_ids(self):
        count = 0
        while count <= 1000:
            count += 1
            span = opentracing.tracer.start_span("test_span_ids")
            context = span.context
            assert 0 <= int(context.span_id, 16) <= 18446744073709551615
            assert 0 <= int(context.trace_id, 16) <= 18446744073709551615

    def test_span_fields(self):
        span = opentracing.tracer.start_span("mycustom")
        self.assertEqual("mycustom", span.operation_name)
        assert span.context

        span.set_tag("tagone", "string")
        span.set_tag("tagtwo", 150)

        self.assertEqual("string", span.tags['tagone'])
        self.assertEqual(150, span.tags['tagtwo'])

    def test_span_queueing(self):
        recorder = opentracing.tracer.recorder

        count = 1
        while count <= 20:
            count += 1
            span = opentracing.tracer.start_span("queuethisplz")
            span.set_tag("tagone", "string")
            span.set_tag("tagtwo", 150)
            span.finish()

        self.assertEqual(20, recorder.queue_size())

    def test_sdk_spans(self):
        recorder = opentracing.tracer.recorder

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag("tagone", "string")
        span.set_tag("tagtwo", 150)
        span.set_tag('span.kind', "entry")
        time.sleep(0.5)
        span.finish()

        spans = recorder.queued_spans()
        assert 1, len(spans)

        sdk_span = spans[0]
        self.assertEqual('sdk', sdk_span.n)
        self.assertEqual(None, sdk_span.p)
        self.assertEqual(sdk_span.s, sdk_span.t)
        assert sdk_span.ts
        assert sdk_span.ts > 0
        assert sdk_span.d
        assert sdk_span.d > 0

        assert sdk_span.data
        assert sdk_span.data["sdk"]
        self.assertEqual('entry', sdk_span.data["sdk"]["type"])
        self.assertEqual('custom_sdk_span', sdk_span.data["sdk"]["name"])
        assert sdk_span.data["sdk"]["custom"]
        assert sdk_span.data["sdk"]["custom"]["tags"]

    def test_span_kind(self):
        recorder = opentracing.tracer.recorder

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "consumer")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "server")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "producer")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "client")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "blah")
        span.finish()

        spans = recorder.queued_spans()
        assert 5, len(spans)

        span = spans[0]
        self.assertEqual('entry', span.data["sdk"]["type"])

        span = spans[1]
        self.assertEqual('entry', span.data["sdk"]["type"])

        span = spans[2]
        self.assertEqual('exit', span.data["sdk"]["type"])

        span = spans[3]
        self.assertEqual('exit', span.data["sdk"]["type"])

        span = spans[4]
        self.assertEqual('intermediate', span.data["sdk"]["type"])

        span = spans[0]
        self.assertEqual(1, span.k)

        span = spans[1]
        self.assertEqual(1, span.k)

        span = spans[2]
        self.assertEqual(2, span.k)

        span = spans[3]
        self.assertEqual(2, span.k)

        span = spans[4]
        self.assertEqual(3, span.k)

    def test_tag_values(self):
        with tracer.start_active_span('test') as scope:
            # Set a UUID class as a tag
            # If unchecked, this causes a json.dumps error: "ValueError: Circular reference detected"
            scope.span.set_tag('uuid', UUID(bytes=b'\x12\x34\x56\x78'*4))
            # Arbitrarily setting an instance of some class
            scope.span.set_tag('tracer', tracer)
            scope.span.set_tag('none', None)
            scope.span.set_tag('mylist', [1, 2, 3])
            scope.span.set_tag('myset', {"one", 2})

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 1

        test_span = spans[0]
        assert(test_span)
        assert(len(test_span.data['sdk']['custom']['tags']) == 5)
        assert(test_span.data['sdk']['custom']['tags']['uuid'] == "UUID('12345678-1234-5678-1234-567812345678')")
        assert(test_span.data['sdk']['custom']['tags']['tracer'])
        assert(test_span.data['sdk']['custom']['tags']['none'] == 'None')
        assert(test_span.data['sdk']['custom']['tags']['mylist'] == [1, 2, 3])
        if PY2:
            set_regexp = re.compile(r"set\(\[.*,.*\]\)")
            assert(set_regexp.search(test_span.data['sdk']['custom']['tags']['myset']))
        else:
            set_regexp = re.compile(r"\{.*,.*\}")
            assert(set_regexp.search(test_span.data['sdk']['custom']['tags']['myset']))

        # Convert to JSON
        json_data = to_json(test_span)
        assert(json_data)

        # And back
        span_dict = json.loads(json_data)
        assert(len(span_dict['data']['sdk']['custom']['tags']) == 5)
        assert(span_dict['data']['sdk']['custom']['tags']['uuid'] == "UUID('12345678-1234-5678-1234-567812345678')")
        assert(span_dict['data']['sdk']['custom']['tags']['tracer'])
        assert(span_dict['data']['sdk']['custom']['tags']['none'] == 'None')
        assert(span_dict['data']['sdk']['custom']['tags']['mylist'] == [1, 2, 3])
        if PY2:
            set_regexp = re.compile(r"set\(\[.*,.*\]\)")
            assert(set_regexp.search(test_span.data['sdk']['custom']['tags']['myset']))
        else:
            set_regexp = re.compile(r"{.*,.*}")
            assert(set_regexp.search(test_span.data['sdk']['custom']['tags']['myset']))

    def test_tag_names(self):
        with tracer.start_active_span('test') as scope:
            # Tag names (keys) must be strings
            scope.span.set_tag(1234567890, 'This should not get set')
            # Unicode key name
            scope.span.set_tag(u'asdf', 'This should be ok')

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 1

        test_span = spans[0]
        assert(test_span)
        assert(len(test_span.data['sdk']['custom']['tags']) == 1)
        assert(test_span.data['sdk']['custom']['tags']['asdf'] == 'This should be ok')

        json_data = to_json(test_span)
        assert(json_data)

    def test_custom_service_name(self):
        # Set a custom service name
        agent.options.service_name = "custom_service_name"

        with tracer.start_active_span('entry_span') as scope:
            scope.span.set_tag('span.kind', 'server')
            scope.span.set_tag(u'type', 'entry_span')

            with tracer.start_active_span('intermediate_span', child_of=scope.span) as exit_scope:
                exit_scope.span.set_tag(u'type', 'intermediate_span')

            with tracer.start_active_span('exit_span', child_of=scope.span) as exit_scope:
                exit_scope.span.set_tag('span.kind', 'client')
                exit_scope.span.set_tag(u'type', 'exit_span')

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 3

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "entry_span"
        entry_span = get_first_span_by_filter(spans, filter)
        assert (entry_span)

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "intermediate_span"
        intermediate_span = get_first_span_by_filter(spans, filter)
        assert (intermediate_span)

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "exit_span"
        exit_span = get_first_span_by_filter(spans, filter)
        assert (exit_span)

        # Custom service name should be set on ENTRY spans and none other
        assert(entry_span)
        assert(len(entry_span.data['sdk']['custom']['tags']) == 2)
        assert(entry_span.data['sdk']['custom']['tags']['type'] == 'entry_span')
        assert(entry_span.data['service'] == 'custom_service_name')
        assert(entry_span.k == 1)

        assert(intermediate_span)
        assert(len(intermediate_span.data['sdk']['custom']['tags']) == 1)
        assert(intermediate_span.data['sdk']['custom']['tags']['type'] == 'intermediate_span')
        assert("service" not in intermediate_span.data)
        assert(intermediate_span.k == 3)

        assert(exit_span)
        assert(len(exit_span.data['sdk']['custom']['tags']) == 2)
        assert(exit_span.data['sdk']['custom']['tags']['type'] == 'exit_span')
        assert("service" not in intermediate_span.data)
        assert(exit_span.k == 2)

    def test_span_log(self):
        with tracer.start_active_span('mylogspan') as scope:
            scope.span.log_kv({'Don McLean': 'American Pie'})
            scope.span.log_kv({'Elton John': 'Your Song'})

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 1

        my_log_span = spans[0]
        assert my_log_span.n == 'sdk'

        log_data = my_log_span.data['sdk']['custom']['logs']
        assert len(log_data) == 2







from __future__ import absolute_import

import unittest

import redis
from ..helpers import testenv
from redis.sentinel import Sentinel
from instana.singletons import tracer


class TestRedis(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()

        # self.sentinel = Sentinel([(testenv['redis_host'], 26379)], socket_timeout=0.1)
        # self.sentinel_master = self.sentinel.discover_master('mymaster')
        # self.client = redis.Redis(host=self.sentinel_master[0])

        self.client = redis.Redis(host=testenv['redis_host'])

    def tearDown(self):
        pass

    def test_vanilla(self):
        self.client.set('instrument', 'piano')
        result = self.client.get('instrument')

    def test_set_get(self):
        result = None
        with tracer.start_active_span('test'):
            self.client.set('foox', 'barX')
            self.client.set('fooy', 'barY')
            result = self.client.get('foox')

        spans = self.recorder.queued_spans()
        self.assertEqual(4, len(spans))

        self.assertEqual(b'barX', result)

        rs1_span = spans[0]
        rs2_span = spans[1]
        rs3_span = spans[2]
        test_span = spans[3]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, rs1_span.t)
        self.assertEqual(test_span.t, rs2_span.t)
        self.assertEqual(test_span.t, rs3_span.t)

        # Parent relationships
        self.assertEqual(rs1_span.p, test_span.s)
        self.assertEqual(rs2_span.p, test_span.s)
        self.assertEqual(rs3_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(rs1_span.ec)
        self.assertIsNone(rs2_span.ec)
        self.assertIsNone(rs3_span.ec)

        # Redis span 1
        self.assertEqual('redis', rs1_span.n)
        self.assertFalse('custom' in rs1_span.data)
        self.assertTrue('redis' in rs1_span.data)

        self.assertEqual('redis-py', rs1_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs1_span.data["redis"]["connection"])
        self.assertEqual("SET", rs1_span.data["redis"]["command"])
        self.assertIsNone(rs1_span.data["redis"]["error"])

        self.assertIsNotNone(rs1_span.stack)
        self.assertTrue(type(rs1_span.stack) is list)
        self.assertGreater(len(rs1_span.stack), 0)

        # Redis span 2
        self.assertEqual('redis', rs2_span.n)
        self.assertFalse('custom' in rs2_span.data)
        self.assertTrue('redis' in rs2_span.data)

        self.assertEqual('redis-py', rs2_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs2_span.data["redis"]["connection"])
        self.assertEqual("SET", rs2_span.data["redis"]["command"])
        self.assertIsNone(rs2_span.data["redis"]["error"])

        self.assertIsNotNone(rs2_span.stack)
        self.assertTrue(type(rs2_span.stack) is list)
        self.assertGreater(len(rs2_span.stack), 0)

        # Redis span 3
        self.assertEqual('redis', rs3_span.n)
        self.assertFalse('custom' in rs3_span.data)
        self.assertTrue('redis' in rs3_span.data)

        self.assertEqual('redis-py', rs3_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs3_span.data["redis"]["connection"])
        self.assertEqual("GET", rs3_span.data["redis"]["command"])
        self.assertIsNone(rs3_span.data["redis"]["error"])

        self.assertIsNotNone(rs3_span.stack)
        self.assertTrue(type(rs3_span.stack) is list)
        self.assertGreater(len(rs3_span.stack), 0)

    def test_set_incr_get(self):
        result = None
        with tracer.start_active_span('test'):
            self.client.set('counter', '10')
            self.client.incr('counter')
            result = self.client.get('counter')

        spans = self.recorder.queued_spans()
        self.assertEqual(4, len(spans))

        self.assertEqual(b'11', result)

        rs1_span = spans[0]
        rs2_span = spans[1]
        rs3_span = spans[2]
        test_span = spans[3]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, rs1_span.t)
        self.assertEqual(test_span.t, rs2_span.t)
        self.assertEqual(test_span.t, rs3_span.t)

        # Parent relationships
        self.assertEqual(rs1_span.p, test_span.s)
        self.assertEqual(rs2_span.p, test_span.s)
        self.assertEqual(rs3_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(rs1_span.ec)
        self.assertIsNone(rs2_span.ec)
        self.assertIsNone(rs3_span.ec)

        # Redis span 1
        self.assertEqual('redis', rs1_span.n)
        self.assertFalse('custom' in rs1_span.data)
        self.assertTrue('redis' in rs1_span.data)

        self.assertEqual('redis-py', rs1_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs1_span.data["redis"]["connection"])
        self.assertEqual("SET", rs1_span.data["redis"]["command"])
        self.assertIsNone(rs1_span.data["redis"]["error"])

        self.assertIsNotNone(rs1_span.stack)
        self.assertTrue(type(rs1_span.stack) is list)
        self.assertGreater(len(rs1_span.stack), 0)

        # Redis span 2
        self.assertEqual('redis', rs2_span.n)
        self.assertFalse('custom' in rs2_span.data)
        self.assertTrue('redis' in rs2_span.data)

        self.assertEqual('redis-py', rs2_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs2_span.data["redis"]["connection"])
        self.assertEqual("INCRBY", rs2_span.data["redis"]["command"])
        self.assertIsNone(rs2_span.data["redis"]["error"])

        self.assertIsNotNone(rs2_span.stack)
        self.assertTrue(type(rs2_span.stack) is list)
        self.assertGreater(len(rs2_span.stack), 0)

        # Redis span 3
        self.assertEqual('redis', rs3_span.n)
        self.assertFalse('custom' in rs3_span.data)
        self.assertTrue('redis' in rs3_span.data)

        self.assertEqual('redis-py', rs3_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs3_span.data["redis"]["connection"])
        self.assertEqual("GET", rs3_span.data["redis"]["command"])
        self.assertIsNone(rs3_span.data["redis"]["error"])

        self.assertIsNotNone(rs3_span.stack)
        self.assertTrue(type(rs3_span.stack) is list)
        self.assertGreater(len(rs3_span.stack), 0)

    def test_old_redis_client(self):
        result = None
        with tracer.start_active_span('test'):
            self.client.set('foox', 'barX')
            self.client.set('fooy', 'barY')
            result = self.client.get('foox')

        spans = self.recorder.queued_spans()
        self.assertEqual(4, len(spans))

        self.assertEqual(b'barX', result)

        rs1_span = spans[0]
        rs2_span = spans[1]
        rs3_span = spans[2]
        test_span = spans[3]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, rs1_span.t)
        self.assertEqual(test_span.t, rs2_span.t)
        self.assertEqual(test_span.t, rs3_span.t)

        # Parent relationships
        self.assertEqual(rs1_span.p, test_span.s)
        self.assertEqual(rs2_span.p, test_span.s)
        self.assertEqual(rs3_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(rs1_span.ec)
        self.assertIsNone(rs2_span.ec)
        self.assertIsNone(rs3_span.ec)

        # Redis span 1
        self.assertEqual('redis', rs1_span.n)
        self.assertFalse('custom' in rs1_span.data)
        self.assertTrue('redis' in rs1_span.data)

        self.assertEqual('redis-py', rs1_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs1_span.data["redis"]["connection"])
        self.assertEqual("SET", rs1_span.data["redis"]["command"])
        self.assertIsNone(rs1_span.data["redis"]["error"])

        self.assertIsNotNone(rs1_span.stack)
        self.assertTrue(type(rs1_span.stack) is list)
        self.assertGreater(len(rs1_span.stack), 0)

        # Redis span 2
        self.assertEqual('redis', rs2_span.n)
        self.assertFalse('custom' in rs2_span.data)
        self.assertTrue('redis' in rs2_span.data)

        self.assertEqual('redis-py', rs2_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs2_span.data["redis"]["connection"])
        self.assertEqual("SET", rs2_span.data["redis"]["command"])
        self.assertIsNone(rs2_span.data["redis"]["error"])

        self.assertIsNotNone(rs2_span.stack)
        self.assertTrue(type(rs2_span.stack) is list)
        self.assertGreater(len(rs2_span.stack), 0)

        # Redis span 3
        self.assertEqual('redis', rs3_span.n)
        self.assertFalse('custom' in rs3_span.data)
        self.assertTrue('redis' in rs3_span.data)

        self.assertEqual('redis-py', rs3_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs3_span.data["redis"]["connection"])
        self.assertEqual("GET", rs3_span.data["redis"]["command"])
        self.assertIsNone(rs3_span.data["redis"]["error"])

        self.assertIsNotNone(rs3_span.stack)
        self.assertTrue(type(rs3_span.stack) is list)
        self.assertGreater(len(rs3_span.stack), 0)

    def test_pipelined_requests(self):
        result = None
        with tracer.start_active_span('test'):
            pipe = self.client.pipeline()
            pipe.set('foox', 'barX')
            pipe.set('fooy', 'barY')
            pipe.get('foox')
            result = pipe.execute()

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        self.assertEqual([True, True, b'barX'], result)

        rs1_span = spans[0]
        test_span = spans[1]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, rs1_span.t)

        # Parent relationships
        self.assertEqual(rs1_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(rs1_span.ec)

        # Redis span 1
        self.assertEqual('redis', rs1_span.n)
        self.assertFalse('custom' in rs1_span.data)
        self.assertTrue('redis' in rs1_span.data)

        self.assertEqual('redis-py', rs1_span.data["redis"]["driver"])
        self.assertEqual("redis://%s:6379/0" % testenv['redis_host'], rs1_span.data["redis"]["connection"])
        self.assertEqual("PIPELINE", rs1_span.data["redis"]["command"])
        self.assertEqual(['SET', 'SET', 'GET'], rs1_span.data["redis"]["subCommands"])
        self.assertIsNone(rs1_span.data["redis"]["error"])

        self.assertIsNotNone(rs1_span.stack)
        self.assertTrue(type(rs1_span.stack) is list)
        self.assertGreater(len(rs1_span.stack), 0)



import threading


def launch_background_thread(app, name):
    app_thread = threading.Thread(target=app)
    app_thread.daemon = True
    app_thread.name = "Background %s app" % name
    print("Starting background %s app..." % name)
    app_thread.start()
    return app_thread



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stan.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='stan.proto',
  package='stan',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\nstan.proto\x12\x04stan\"#\n\x0fQuestionRequest\x12\x10\n\x08question\x18\x01 \x01(\t\"8\n\x10QuestionResponse\x12\x0e\n\x06\x61nswer\x18\x01 \x01(\t\x12\x14\n\x0cwas_answered\x18\x02 \x01(\x08\x32\xe3\x03\n\x04Stan\x12I\n\x16OneQuestionOneResponse\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00\x12M\n\x18ManyQuestionsOneResponse\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00(\x01\x12M\n\x18OneQuestionManyResponses\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00\x30\x01\x12P\n\x19ManyQuestionsManyReponses\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00(\x01\x30\x01\x12N\n\x1bOneQuestionOneErrorResponse\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00\x12P\n\x1dOneErroredQuestionOneResponse\x12\x15.stan.QuestionRequest\x1a\x16.stan.QuestionResponse\"\x00\x62\x06proto3')
)




_QUESTIONREQUEST = _descriptor.Descriptor(
  name='QuestionRequest',
  full_name='stan.QuestionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='question', full_name='stan.QuestionRequest.question', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=55,
)


_QUESTIONRESPONSE = _descriptor.Descriptor(
  name='QuestionResponse',
  full_name='stan.QuestionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='answer', full_name='stan.QuestionResponse.answer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='was_answered', full_name='stan.QuestionResponse.was_answered', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=57,
  serialized_end=113,
)

DESCRIPTOR.message_types_by_name['QuestionRequest'] = _QUESTIONREQUEST
DESCRIPTOR.message_types_by_name['QuestionResponse'] = _QUESTIONRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

QuestionRequest = _reflection.GeneratedProtocolMessageType('QuestionRequest', (_message.Message,), dict(
  DESCRIPTOR = _QUESTIONREQUEST,
  __module__ = 'stan_pb2'
  # @@protoc_insertion_point(class_scope:stan.QuestionRequest)
  ))
_sym_db.RegisterMessage(QuestionRequest)

QuestionResponse = _reflection.GeneratedProtocolMessageType('QuestionResponse', (_message.Message,), dict(
  DESCRIPTOR = _QUESTIONRESPONSE,
  __module__ = 'stan_pb2'
  # @@protoc_insertion_point(class_scope:stan.QuestionResponse)
  ))
_sym_db.RegisterMessage(QuestionResponse)



_STAN = _descriptor.ServiceDescriptor(
  name='Stan',
  full_name='stan.Stan',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=116,
  serialized_end=599,
  methods=[
  _descriptor.MethodDescriptor(
    name='OneQuestionOneResponse',
    full_name='stan.Stan.OneQuestionOneResponse',
    index=0,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ManyQuestionsOneResponse',
    full_name='stan.Stan.ManyQuestionsOneResponse',
    index=1,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OneQuestionManyResponses',
    full_name='stan.Stan.OneQuestionManyResponses',
    index=2,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ManyQuestionsManyReponses',
    full_name='stan.Stan.ManyQuestionsManyReponses',
    index=3,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OneQuestionOneErrorResponse',
    full_name='stan.Stan.OneQuestionOneErrorResponse',
    index=4,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OneErroredQuestionOneResponse',
    full_name='stan.Stan.OneErroredQuestionOneResponse',
    index=5,
    containing_service=None,
    input_type=_QUESTIONREQUEST,
    output_type=_QUESTIONRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STAN)

DESCRIPTOR.services_by_name['Stan'] = _STAN

# @@protoc_insertion_point(module_scope)




class SpanContext():
    def __init__(
            self,
            trace_id=None,
            span_id=None,
            baggage=None,
            sampled=True,
            level=1,
            synthetic=False):

        self.level = level
        self.trace_id = trace_id
        self.span_id = span_id
        self.sampled = sampled
        self.synthetic = synthetic
        self._baggage = baggage or {}

    @property
    def baggage(self):
        return self._baggage

    def with_baggage_item(self, key, value):
        new_baggage = self._baggage.copy()
        new_baggage[key] = value
        return SpanContext(
            trace_id=self.trace_id,
            span_id=self.span_id,
            sampled=self.sampled,
            baggage=new_baggage)


from __future__ import absolute_import

import os
import json
import unittest

from instana.tracer import InstanaTracer
from instana.recorder import StanRecorder
from instana.agent.aws_fargate import AWSFargateAgent
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer


def get_docker_plugin(plugins):
    """
    Given a list of plugins, find and return the docker plugin that we're interested in from the mock data
    """
    docker_plugin = None
    for plugin in plugins:
        if plugin["name"] == "com.instana.plugin.docker" and plugin["entityId"] == "arn:aws:ecs:us-east-2:410797082306:task/2d60afb1-e7fd-4761-9430-a375293a9b82::docker-ssh-aws-fargate":
            docker_plugin = plugin
    return docker_plugin


class TestFargateCollector(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestFargateCollector, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None
        self.pwd = os.path.dirname(os.path.realpath(__file__))

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        os.environ["AWS_EXECUTION_ENV"] = "AWS_ECS_FARGATE"
        os.environ["INSTANA_ENDPOINT_URL"] = "https://localhost/notreal"
        os.environ["INSTANA_AGENT_KEY"] = "Fake_Key"

        if "INSTANA_ZONE" in os.environ:
            os.environ.pop("INSTANA_ZONE")
        if "INSTANA_TAGS" in os.environ:
            os.environ.pop("INSTANA_TAGS")

    def tearDown(self):
        """ Reset all environment variables of consequence """
        if "AWS_EXECUTION_ENV" in os.environ:
            os.environ.pop("AWS_EXECUTION_ENV")
        if "INSTANA_EXTRA_HTTP_HEADERS" in os.environ:
            os.environ.pop("INSTANA_EXTRA_HTTP_HEADERS")
        if "INSTANA_ENDPOINT_URL" in os.environ:
            os.environ.pop("INSTANA_ENDPOINT_URL")
        if "INSTANA_AGENT_KEY" in os.environ:
            os.environ.pop("INSTANA_AGENT_KEY")
        if "INSTANA_ZONE" in os.environ:
            os.environ.pop("INSTANA_ZONE")
        if "INSTANA_TAGS" in os.environ:
            os.environ.pop("INSTANA_TAGS")

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = AWSFargateAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

        # Manually set the ECS Metadata API results on the collector
        with open(self.pwd + '/../data/fargate/1.3.0/root_metadata.json', 'r') as json_file:
            self.agent.collector.root_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/task_metadata.json', 'r') as json_file:
            self.agent.collector.task_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/stats_metadata.json', 'r') as json_file:
            self.agent.collector.stats_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/task_stats_metadata.json', 'r') as json_file:
            self.agent.collector.task_stats_metadata = json.load(json_file)

    def test_prepare_payload_basics(self):
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        assert(payload)

        assert(len(payload.keys()) == 2)
        assert('spans' in payload)
        assert(isinstance(payload['spans'], list))
        assert(len(payload['spans']) == 0)
        assert('metrics' in payload)
        assert(len(payload['metrics'].keys()) == 1)
        assert('plugins' in payload['metrics'])
        assert(isinstance(payload['metrics']['plugins'], list))
        assert(len(payload['metrics']['plugins']) == 7)

        plugins = payload['metrics']['plugins']
        for plugin in plugins:
            # print("%s - %s" % (plugin["name"], plugin["entityId"]))
            assert('name' in plugin)
            assert('entityId' in plugin)
            assert('data' in plugin)

    def test_docker_plugin_snapshot_data(self):
        self.create_agent_and_setup_tracer()

        first_payload = self.agent.collector.prepare_payload()
        second_payload = self.agent.collector.prepare_payload()

        assert(first_payload)
        assert(second_payload)

        plugin_first_report = get_docker_plugin(first_payload['metrics']['plugins'])
        plugin_second_report = get_docker_plugin(second_payload['metrics']['plugins'])

        assert(plugin_first_report)
        assert("data" in plugin_first_report)

        # First report should have snapshot data
        data = plugin_first_report["data"]
        assert(data["Id"] == "63dc7ac9f3130bba35c785ed90ff12aad82087b5c5a0a45a922c45a64128eb45")
        assert(data["Created"] == "2020-07-27T12:14:12.583114444Z")
        assert(data["Started"] == "2020-07-27T12:14:13.545410186Z")
        assert(data["Image"] == "410797082306.dkr.ecr.us-east-2.amazonaws.com/fargate-docker-ssh:latest")
        assert(data["Labels"] == {'com.amazonaws.ecs.cluster': 'arn:aws:ecs:us-east-2:410797082306:cluster/lombardo-ssh-cluster', 'com.amazonaws.ecs.container-name': 'docker-ssh-aws-fargate', 'com.amazonaws.ecs.task-arn': 'arn:aws:ecs:us-east-2:410797082306:task/2d60afb1-e7fd-4761-9430-a375293a9b82', 'com.amazonaws.ecs.task-definition-family': 'docker-ssh-aws-fargate', 'com.amazonaws.ecs.task-definition-version': '1'})
        assert(data["Ports"] is None)

        # Second report should have no snapshot data
        assert(plugin_second_report)
        assert("data" in plugin_second_report)
        data = plugin_second_report["data"]
        assert("Id" in data)
        assert("Created" not in data)
        assert("Started" not in data)
        assert("Image" not in data)
        assert("Labels" not in data)
        assert("Ports" not in data)

    def test_docker_plugin_metrics(self):
        self.create_agent_and_setup_tracer()

        first_payload = self.agent.collector.prepare_payload()
        second_payload = self.agent.collector.prepare_payload()

        assert(first_payload)
        assert(second_payload)

        plugin_first_report = get_docker_plugin(first_payload['metrics']['plugins'])
        assert(plugin_first_report)
        assert("data" in plugin_first_report)

        plugin_second_report = get_docker_plugin(second_payload['metrics']['plugins'])
        assert(plugin_second_report)
        assert("data" in plugin_second_report)

        # First report should report all metrics
        data = plugin_first_report.get("data", None)
        assert(data)
        assert "network" not in data

        cpu = data.get("cpu", None)
        assert(cpu)
        assert(cpu["total_usage"] == 0.011033)
        assert(cpu["user_usage"] == 0.009918)
        assert(cpu["system_usage"] == 0.00089)
        assert(cpu["throttling_count"] == 0)
        assert(cpu["throttling_time"] == 0)

        memory = data.get("memory", None)
        assert(memory)
        assert(memory["active_anon"] == 78721024)
        assert(memory["active_file"] == 18501632)
        assert(memory["inactive_anon"] == 0)
        assert(memory["inactive_file"] == 71684096)
        assert(memory["total_cache"] == 90185728)
        assert(memory["total_rss"] == 78721024)
        assert(memory["usage"] == 193769472)
        assert(memory["max_usage"] == 195305472)
        assert(memory["limit"] == 536870912)

        blkio = data.get("blkio", None)
        assert(blkio)
        assert(blkio["blk_read"] == 0)
        assert(blkio["blk_write"] == 128352256)

        # Second report should report the delta (in the test case, nothing)
        data = plugin_second_report["data"]
        assert("cpu" in data)
        assert(len(data["cpu"]) == 0)
        assert("memory" in data)
        assert(len(data["memory"]) == 0)
        assert("blkio" in data)
        assert(len(data["blkio"]) == 1)
        assert(data["blkio"]['blk_write'] == 0)
        assert('blk_read' not in data["blkio"])

    def test_no_instana_zone(self):
        self.create_agent_and_setup_tracer()
        assert(self.agent.options.zone is None)

    def test_instana_zone(self):
        os.environ["INSTANA_ZONE"] = "YellowDog"
        self.create_agent_and_setup_tracer()

        assert(self.agent.options.zone == "YellowDog")

        payload = self.agent.collector.prepare_payload()
        assert(payload)

        plugins = payload['metrics']['plugins']
        assert(isinstance(plugins, list))

        task_plugin = None
        for plugin in plugins:
            if plugin["name"] == "com.instana.plugin.aws.ecs.task":
                task_plugin = plugin

        assert(task_plugin)
        assert("data" in task_plugin)
        assert("instanaZone" in task_plugin["data"])
        assert(task_plugin["data"]["instanaZone"] == "YellowDog")

    def test_custom_tags(self):
        os.environ["INSTANA_TAGS"] = "love,war=1,games"
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent.options, 'tags'))
        self.assertEqual(self.agent.options.tags, {"love": None, "war": "1", "games": None})

        payload = self.agent.collector.prepare_payload()

        assert payload
        task_plugin = None
        plugins = payload['metrics']['plugins']
        for plugin in plugins:
            if plugin["name"] == "com.instana.plugin.aws.ecs.task":
                task_plugin = plugin
        assert task_plugin
        assert "tags" in task_plugin["data"]
        tags = task_plugin["data"]["tags"]
        assert tags["war"] == "1"
        assert tags["love"] is None
        assert tags["games"] is None



from __future__ import absolute_import

from ..log import logger
from .pep0249 import ConnectionFactory

try:
    import MySQLdb

    cf = ConnectionFactory(connect_func=MySQLdb.connect, module_name='mysql')

    setattr(MySQLdb, 'connect', cf)
    if hasattr(MySQLdb, 'Connect'):
        setattr(MySQLdb, 'Connect', cf)

    logger.debug("Instrumenting mysqlclient")
except ImportError:
    pass



from __future__ import absolute_import

import os
import logging
import unittest

from instana.agent.host import HostAgent
from instana.tracer import InstanaTracer
from instana.options import StandardOptions
from instana.recorder import StanRecorder
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer


class TestHost(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestHost, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        pass

    def tearDown(self):
        """ Reset all environment variables of consequence """
        if "AWS_EXECUTION_ENV" in os.environ:
            os.environ.pop("AWS_EXECUTION_ENV")
        if "INSTANA_EXTRA_HTTP_HEADERS" in os.environ:
            os.environ.pop("INSTANA_EXTRA_HTTP_HEADERS")
        if "INSTANA_ENDPOINT_URL" in os.environ:
            os.environ.pop("INSTANA_ENDPOINT_URL")
        if "INSTANA_ENDPOINT_PROXY" in os.environ:
            os.environ.pop("INSTANA_ENDPOINT_PROXY")
        if "INSTANA_AGENT_KEY" in os.environ:
            os.environ.pop("INSTANA_AGENT_KEY")
        if "INSTANA_LOG_LEVEL" in os.environ:
            os.environ.pop("INSTANA_LOG_LEVEL")
        if "INSTANA_SERVICE_NAME" in os.environ:
            os.environ.pop("INSTANA_SERVICE_NAME")
        if "INSTANA_SECRETS" in os.environ:
            os.environ.pop("INSTANA_SECRETS")
        if "INSTANA_TAGS" in os.environ:
            os.environ.pop("INSTANA_TAGS")

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = HostAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

    def test_secrets(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent.options, 'secrets_matcher'))
        self.assertEqual(self.agent.options.secrets_matcher, 'contains-ignore-case')
        self.assertTrue(hasattr(self.agent.options, 'secrets_list'))
        self.assertEqual(self.agent.options.secrets_list, ['key', 'pass', 'secret'])

    def test_options_have_extra_http_headers(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(hasattr(self.agent.options, 'extra_http_headers'))

    def test_has_options(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(isinstance(self.agent.options, StandardOptions))

    def test_agent_default_log_level(self):
        self.create_agent_and_setup_tracer()
        assert self.agent.options.log_level == logging.WARNING

    def test_agent_instana_debug(self):
        os.environ['INSTANA_DEBUG'] = "asdf"
        self.create_agent_and_setup_tracer()
        assert self.agent.options.log_level == logging.DEBUG
    
    def test_agent_instana_service_name(self):
        os.environ['INSTANA_SERVICE_NAME'] = "greycake"
        self.create_agent_and_setup_tracer()
        assert self.agent.options.service_name == "greycake"



"""
The Instana agent (for AWS Fargate) that manages
monitoring state and reporting that data.
"""
import time
from instana.options import AWSFargateOptions
from instana.collector.aws_fargate import AWSFargateCollector
from ..log import logger
from ..util import to_json
from .base import BaseAgent
from ..version import VERSION


class AWSFargateFrom(object):
    """ The source identifier for AWSFargateAgent """
    hl = True
    cp = "aws"
    e = "taskDefinition"

    def __init__(self, **kwds):
        self.__dict__.update(kwds)


class AWSFargateAgent(BaseAgent):
    """ In-process agent for AWS Fargate """
    def __init__(self):
        super(AWSFargateAgent, self).__init__()

        self.options = AWSFargateOptions()
        self.from_ = AWSFargateFrom()
        self.collector = None
        self.report_headers = None
        self._can_send = False

        # Update log level (if INSTANA_LOG_LEVEL was set)
        self.update_log_level()

        logger.info("Stan is on the AWS Fargate scene.  Starting Instana instrumentation version: %s", VERSION)

        if self._validate_options():
            self._can_send = True
            self.collector = AWSFargateCollector(self)
            self.collector.start()
        else:
            logger.warning("Required INSTANA_AGENT_KEY and/or INSTANA_ENDPOINT_URL environment variables not set.  "
                           "We will not be able monitor this AWS Fargate cluster.")

    def can_send(self):
        """
        Are we in a state where we can send data?
        @return: Boolean
        """
        return self._can_send

    def get_from_structure(self):
        """
        Retrieves the From data that is reported alongside monitoring data.
        @return: dict()
        """
        return {'hl': True, 'cp': 'aws', 'e': self.collector.get_fq_arn()}

    def report_data_payload(self, payload):
        """
        Used to report metrics and span data to the endpoint URL in self.options.endpoint_url
        """
        response = None
        try:
            if self.report_headers is None:
                # Prepare request headers
                self.report_headers = dict()
                self.report_headers["Content-Type"] = "application/json"
                self.report_headers["X-Instana-Host"] = self.collector.get_fq_arn()
                self.report_headers["X-Instana-Key"] = self.options.agent_key

            self.report_headers["X-Instana-Time"] = str(round(time.time() * 1000))

            response = self.client.post(self.__data_bundle_url(),
                                        data=to_json(payload),
                                        headers=self.report_headers,
                                        timeout=self.options.timeout,
                                        verify=self.options.ssl_verify,
                                        proxies=self.options.endpoint_proxy)

            if not 200 <= response.status_code < 300:
                logger.info("report_data_payload: Instana responded with status code %s", response.status_code)
        except Exception as exc:
            logger.debug("report_data_payload: connection error (%s)", type(exc))
        return response

    def _validate_options(self):
        """
        Validate that the options used by this Agent are valid.  e.g. can we report data?
        """
        return self.options.endpoint_url is not None and self.options.agent_key is not None

    def __data_bundle_url(self):
        """
        URL for posting metrics to the host agent.  Only valid when announced.
        """
        return "%s/bundle" % self.options.endpoint_url



import os
import sys
from ...helpers import testenv
from ..utils import launch_background_thread

app_thread = None

if app_thread is None and sys.version_info >= (3, 5, 3) and 'GEVENT_TEST' not in os.environ and 'CASSANDRA_TEST' not in os.environ:
    testenv["tornado_port"] = 10813
    testenv["tornado_server"] = ("http://127.0.0.1:" + str(testenv["tornado_port"]))

    # Background Tornado application
    from .app import run_server

    app_thread = launch_background_thread(run_server, "Tornado")

